//
//  ImagePickerCollectionView.swift
//  image_picker_ios
//
//  Created by Seifeddine Bouzid on 04.05.23.
//

import Foundation
import Photos

class ImagePickerCollectionView: UIViewController {
    
    static func pickImagesFromPhotosApp(limit: Int, resultProtocol: ImagePickerResult?) {
        let customPicker = ImagePickerCollectionView()
        customPicker.resultProtocol = resultProtocol
        customPicker.limit = limit
        self.requestAuthGallery { valid in
            if valid {
                DispatchQueue.main.async {
                    if let viewController = (UIApplication.shared.delegate as? FlutterAppDelegate)?.window?.rootViewController {
                        viewController.present(customPicker, animated: true, completion: nil)
                    } 
                }
            }
        }
    }
    
    private static func requestAuthGallery(completion: @escaping (Bool) -> Void) {
        let status = PHPhotoLibrary.authorizationStatus()
        if status == .denied || status == .restricted {
            let alert = UIAlertController(title: "Photo Library Access Denied",
                                          message: "Please grant permission to access the photo library in the Settings app.",
                                          preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "Cancel", style: .cancel))
            alert.addAction(UIAlertAction(title: "Settings", style: .default,
                                          handler: { _ in
                UIApplication.shared.open(URL(string: UIApplication.openSettingsURLString)!,
                                          options: [:],
                                          completionHandler: nil)
            }))
            if let viewController = (UIApplication.shared.delegate as? FlutterAppDelegate)?.window?.rootViewController {
                viewController.present(alert, animated: true, completion: nil)
                completion(false)
            }
        } else if status == .notDetermined || status == .authorized {
            PHPhotoLibrary.requestAuthorization { status in
                if status == .authorized {
                    completion(true)
                } else {
                    let alert = UIAlertController(title: "Photo Library Access Denied",
                                                  message: "Please grant permission to access the photo library in the Settings app.",
                                                  preferredStyle: .alert)
                    alert.addAction(UIAlertAction(title: "Cancel", style: .cancel))
                    alert.addAction(UIAlertAction(title: "Settings", style: .default,
                                                  handler: { _ in
                        UIApplication.shared.open(URL(string: UIApplication.openSettingsURLString)!,
                                                  options: [:],
                                                  completionHandler: nil)
                    }))
                    if let viewController = (UIApplication.shared.delegate as? FlutterAppDelegate)?.window?.rootViewController {
                        DispatchQueue.main.async {
                            viewController.present(alert, animated: true, completion: nil)
                            completion(false)
                        }
                    }  
                }
            }
        }
        
    }
    
    private lazy var collectionViewLayout: UICollectionViewLayout = { [unowned self] in
        let layout = UICollectionViewFlowLayout()
        layout.scrollDirection = .vertical 
        layout.minimumInteritemSpacing = 2
        layout.minimumLineSpacing = 4
        layout.sectionInset = UIEdgeInsets.zero
        
        return layout
    }()
    
    private lazy var collectionView: UICollectionView = { [unowned self] in
        let collectionView = UICollectionView(frame: CGRect.zero,
                                              collectionViewLayout: self.collectionViewLayout)
        collectionView.translatesAutoresizingMaskIntoConstraints = false
        collectionView.backgroundColor = UIColor.white
        collectionView.showsHorizontalScrollIndicator = false
        collectionView.dataSource = self
        collectionView.delegate = self
        
        return collectionView
    }()
    
    private lazy var doneButton: UIButton = { [unowned self] in 
        let button = UIButton(type: .custom)
        button.translatesAutoresizingMaskIntoConstraints = false
        button.setTitle("Done", for: .normal)
        button.setTitleColor(.systemBlue, for: .normal)
        button.setTitleColor(.systemBlue.withAlphaComponent(0.5), for: .highlighted)
        button.setTitleColor(.systemGray, for: .disabled)
        button.addTarget(self, action: #selector(self.doneButtonAction(_:)), for: .touchUpInside)
        button.isEnabled = false
        return button
    }()
    
    private lazy var cancelButton: UIButton = { [unowned self] in 
        let button = UIButton(frame: CGRect.zero)
        button.translatesAutoresizingMaskIntoConstraints = false
        button.setTitle("Cancel", for: .normal)
        button.setTitleColor(.systemBlue, for: .normal)
        button.setTitleColor(.systemBlue.withAlphaComponent(0.5), for: .highlighted)
        button.addTarget(self, action: #selector(self.cancelButtonAction(_:)), for: .touchUpInside)
        return button
    }()
    
    private lazy var topView: UIView = { [unowned self] in 
        let view = UIView(frame: CGRect.zero)
        view.translatesAutoresizingMaskIntoConstraints = false
        view.backgroundColor = .white
        return view
    }()
    
    private var limit = 0
    private var resultProtocol: ImagePickerResult?
    private var assets : [PHAsset] = []
    private var selectedAssets : [PHAsset] = []
    
    @objc private func cancelButtonAction(_ button: UIButton) {
        self.resultProtocol?.didCancelPickingImage()
        self.dismiss(animated: true)
    }
    
    @objc private func doneButtonAction(_ button: UIButton) {
        self.sendResults()
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.configureTopView()
        self.configureCollectionView()
        self.fetchAllAssets()
    }
    
    private func configureCollectionView() {
        collectionView.register(ImagePickerCollectionCell.self, forCellWithReuseIdentifier: "Cell")
        collectionView.translatesAutoresizingMaskIntoConstraints = false
        self.view.addSubview(collectionView)
        NSLayoutConstraint.activate([
            NSLayoutConstraint(item: collectionView,
                               attribute: .top,
                               relatedBy: .equal,
                               toItem: self.view,
                               attribute: .top,
                               multiplier: 1,
                               constant: 60),
            NSLayoutConstraint(item: collectionView,
                               attribute: .bottom,
                               relatedBy: .equal,
                               toItem: self.view,
                               attribute: .bottom,
                               multiplier: 1,
                               constant: 0),
            NSLayoutConstraint(item: collectionView,
                               attribute: .leading,
                               relatedBy: .equal,
                               toItem: self.view,
                               attribute: .leading,
                               multiplier: 1,
                               constant: 0),
            NSLayoutConstraint(item: collectionView,
                               attribute: .trailing,
                               relatedBy: .equal,
                               toItem: self.view,
                               attribute: .trailing,
                               multiplier: 1,
                               constant: 0),
        ])
    }
    
    private func configureTopView() {
        self.view.addSubview(topView)
        NSLayoutConstraint.activate([
            NSLayoutConstraint(item: topView,
                               attribute: .top,
                               relatedBy: .equal,
                               toItem: self.view,
                               attribute: .top,
                               multiplier: 1,
                               constant: 0),
            NSLayoutConstraint(item: topView,
                               attribute: .leading,
                               relatedBy: .equal,
                               toItem: self.view,
                               attribute: .leading,
                               multiplier: 1,
                               constant: 0),
            NSLayoutConstraint(item: topView,
                               attribute: .trailing,
                               relatedBy: .equal,
                               toItem: self.view,
                               attribute: .trailing,
                               multiplier: 1,
                               constant: 0),
            NSLayoutConstraint(item: topView, 
                               attribute: .height,
                               relatedBy: .equal,
                               toItem: nil, 
                               attribute: .notAnAttribute,
                               multiplier: 1,
                               constant: 60)
        ])
        
        if limit > 1 || limit == 0{
            topView.addSubview(doneButton)
            NSLayoutConstraint.activate([
                NSLayoutConstraint(item: doneButton,
                                   attribute: .trailing,
                                   relatedBy: .equal,
                                   toItem: topView,
                                   attribute: .trailing,
                                   multiplier: 1,
                                   constant: -16),
                NSLayoutConstraint(item: doneButton,
                                   attribute: .centerY,
                                   relatedBy: .equal,
                                   toItem: topView,
                                   attribute: .centerY,
                                   multiplier: 1, 
                                   constant: 0)
            ])
        }
        topView.addSubview(cancelButton)
        
        NSLayoutConstraint.activate([
            NSLayoutConstraint(item: cancelButton,
                               attribute: .leading,
                               relatedBy: .equal,
                               toItem: topView, 
                               attribute: .leading,
                               multiplier: 1,
                               constant: 16),
            NSLayoutConstraint(item: cancelButton,
                               attribute: .centerY,
                               relatedBy: .equal,
                               toItem: topView,
                               attribute: .centerY,
                               multiplier: 1,
                               constant: 0)
        ])  
    }
    
    private func fetchAllAssets() {
        ImagePickerHelper.fetchAssets { assets in
            self.assets = assets
            self.collectionView.reloadData()
        }
    }
    
    private func sendResults() {
        DispatchQueue.global(qos: .background).async {
            var resultData: Dictionary<String,Any> = [String:Any]()
            let paths = self.getAbsoluteUrl(assets: self.selectedAssets)
            let emptyItemsCount = paths.filter({ $0 == "" }).count
            
            if emptyItemsCount > 0 {
                var message = ""
                
                if emptyItemsCount == self.selectedAssets.count {
                    message = "Unable to select the image(s)."
                } else {
                    message = "Unable to select at least one of the images."
                }
                
                resultData["message"] = message
            }
            resultData["uris"] = paths
            DispatchQueue.main.async {
                // main thread
                self.resultProtocol?.didFinishPickingImage(jsonString: resultData.jsonString!)
                self.dismiss(animated: true, completion: nil)
            }
        }
    }
    
    private func getAbsoluteUrl(assets: [PHAsset]) -> [String] {
        var paths: [String] = []
        
        let group = DispatchGroup()
        var path = ""
        for asset in assets {
            group.enter()
            asset.requestContentEditingInput(with: PHContentEditingInputRequestOptions(),
                                             completionHandler: {(data,args) in
                path = data?.fullSizeImageURL?.path ?? ""
                group.leave()
            })
            group.wait()
            paths.append(path)
        }
        return paths
    }
}

extension ImagePickerCollectionView: UICollectionViewDelegateFlowLayout {
    func collectionView(_ collectionView: UICollectionView,
                        layout collectionViewLayout: UICollectionViewLayout,
                        sizeForItemAt indexPath: IndexPath) -> CGSize {
        let cellSize = self.collectionView.bounds.width/3 - 4
        return CGSize(width: cellSize, height: cellSize)
    }
}

extension ImagePickerCollectionView: UICollectionViewDelegate, UICollectionViewDataSource {
    
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return assets.count 
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        guard let cell = collectionView.dequeueReusableCell(withReuseIdentifier: "Cell",
                                                            for: indexPath) as? ImagePickerCollectionCell else {
            return UICollectionViewCell()
        }
        ImagePickerHelper.getImageFromAsset(assets[indexPath.row]) { img in
            cell.configureCell(image: img)
            if self.selectedAssets.contains(where: { selected in
                selected == self.assets[indexPath.row]
            }) {
                cell.check()
            } else {
                cell.unCheck()
            }
        }
        return cell
    }
    
    func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        guard let cell = collectionView.cellForItem(at: indexPath) as? ImagePickerCollectionCell else { return }
        if limit == 1 {
            self.selectedAssets = []
            guard let visibleCells = collectionView.visibleCells as? [ImagePickerCollectionCell] else { return }
            for cell in visibleCells where cell.selectedImageView.image != nil {
                UIView.animate(withDuration: 0.3, animations: {
                    cell.selectedImageView.transform = CGAffineTransform(scaleX: 0.1, y: 0.1)
                }, completion: { _ in
                    cell.unCheck()
                })
            }
        }
        let assetToCheck = assets[indexPath.row]
        if cell.selectedImageView.image != nil {
            UIView.animate(withDuration: 0.3, animations: {
                cell.selectedImageView.transform = CGAffineTransform(scaleX: 0.1, y: 0.1)
            }, completion: { _ in
                cell.unCheck()
            })
            self.selectedAssets.removeAll { asset in
                asset == assetToCheck
            }
        } else if self.limit > self.selectedAssets.count || self.limit == 0 {
            cell.check()
            cell.selectedImageView.transform = CGAffineTransform(scaleX: 0, y: 0)
            UIView.animate(withDuration: 0.3, animations: {
                cell.selectedImageView.transform = CGAffineTransform.identity
            })
            self.selectedAssets.append(assetToCheck)
        }
        if limit == 1 {
            self.sendResults()
        } else {
            self.doneButton.isEnabled = self.selectedAssets.count > 0
        }
    }
}

/** Extension added for json conversion*/
extension Dictionary<String,Any> {
    
    var jsonString: String? {
        if let json = try? JSONSerialization.data(withJSONObject: self) {
            let convertedString = String(data: json, encoding: String.Encoding.utf8)
            return convertedString
        }
        return nil
    }
}
