import 'dart:io';
import 'dart:async';
import 'dart:convert';
import 'package:flutter/services.dart';
import 'package:image_picker/image_picker.dart';
import 'models/image_picker_response.dart';

class ScanbotImagePickerFlutter {
  static const MethodChannel _channel =
      MethodChannel('scanbot_image_picker_flutter');

  /// pick images from gallery and get the uris list
  static Future<ImagePickerResponse> pickImageAsync() async {
    var response = await _pickImagesAsync(1);
    return ImagePickerResponse.withImageResponse(
        uri: response.uris?.first ?? "", message: response.message);
  }

  /// pick images from gallery and get the uris list
  static Future<ImagePickerResponse> pickImagesAsync() async {
    return await _pickImagesAsync(0);
  }

  /// Get the platform version from native side.
  static Future<String?> get platformVersion async {
    final String? version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  /// if version is less than iOS 14
  /// for Android return false always
  /// true: version is less than 14
  /// false: version is 14 or above
  static Future<bool> isVersionLessThanIOSFourteen() async {
    var isVersionLess = false;
    try {
      var result = await _channel.invokeMethod('versionLessThanIOSFourteen');
      if (result.toString() == "true") {
        isVersionLess = true;
      }
    } catch (e) {
      isVersionLess = false;
    }
    return isVersionLess;
  }

  /// pick images from gallery and get the uris list
  static Future<ImagePickerResponse> _pickImagesAsync(int limit) async {
    ImagePickerResponse response;
    if (Platform.isIOS && await isVersionLessThanIOSFourteen()) {
      response = await _pickImagesFromFlutterChannelAsync(limit);
    } else {
      var uris = await _pickImagesFromFlutterImagePicker(limit);
      response =
          ImagePickerResponse.withImagesResponse(uris: uris, message: "");
    }
    return response;
  }

  /// pick images from the common flutter image picker.
  static Future<List<String>> _pickImagesFromFlutterImagePicker(
      int limit) async {
    var paths = List<String>.empty(growable: true);
    if (limit == 1) {
      // Single image selection
      var image = await ImagePicker().pickImage(
          source: ImageSource.gallery,
          maxWidth: double.infinity,
          maxHeight: double.infinity,
          imageQuality: 75);
      paths.add(image?.path ?? "");
      return paths;
    }
    // Multiple images selection
    var images = await ImagePicker().pickMultiImage(
      maxWidth: double.infinity,
      maxHeight: double.infinity,
      imageQuality: 75,
    );
    if (images == null || images.isEmpty) {
      return paths;
    } else {
      for (var image in images) {
        paths.add(image.path);
      }
    }
    return paths;
  }

  /// Pick photos from the library using native code through method channel.
  /// This channel is specifically used only for iOS version below 14.
  static Future<ImagePickerResponse> _pickImagesFromFlutterChannelAsync(
      int limit) async {
    try {
      var result = await _channel
          .invokeMethod('pickImagesFromPhotosApp', {"limit": limit});
      var resultData = ImagePickerResponse.fromJson(jsonDecode(result));
      return resultData;
    } catch (e) {
      return ImagePickerResponse.withImagesResponse(
          uris: List.empty(), message: "Exception retrieving images");
    }
  }
}
